/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package zswzg.kex.tls;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import zswzg.kex.tls.anzeigenglobal.Anzeige;
import zswzg.kex.tls.anzeigenglobal.AnzeigeQuerschnitt;
import zswzg.kex.tls.anzeigenglobal.Dirigent;
import zswzg.kex.tls.kex.KexInputAnzeige;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse der Applikation ZwischenschichtAq.<br>
 * <br>
 * Die Applikation ZwischenschichtAq stellt die Verbindung zwischen dem Modell der KExTls und dem Modell AnzeigeGlobal her.
 * Damit die Applikation erfolgreich arbeiten kann, muss das Modell tmAnzeigenGlobalZwischenschichtTls sinnvoll versorgt sein.
 * <br>
 * <br>
 * Folgende Startparameter sind möglich:<br>
 * <br>
 * DAF-Startparameter wie in SSB beschrieben und durch StandardApplication verarbeitet:<br>
 * <br>
 * Beispiele:<br>
 * -datenverteiler=rechnerDav:8083<br>
 * -debugLevelStdErrText=FINE<br>
 * -benutzer=Tester<br>
 * -authentifizierung=C:\kernsoftware-system-20-03-2006\passwd<br>
 * <br>
 * Spezielle Startparameter:<br>
 * <br>
 * -bereiche=zu verwendende Konfigurationsbereiche durch ':' getrennt (optional, bei keiner Angabe Verwendung aller vorhandenen Konfigurationsbereiche)<br>
 * <br>
 * Beispiele:<br>
 * -bereiche=kb.objekte.VerkehrMqFs.uz.Dambach
 * <br>
 *
 * @version $Revision: 1.5 $ / $Date: 2012/03/09 10:55:21 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class ZwischenschichtAq implements StandardApplication, ClientReceiverInterface
{
  /**
   * DebugLogger für Debug-Ausgaben.
   */
  private static Debug m_debug;
  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface m_connection = null;
  /**
   * Datenmodell.
   */
  private DataModel m_dataModel;
  /**
   * -bereiche von Kommandozeile
   */
  private String _kb;
  /**
   * Liste mit einzelnen Konfigurationsbereichen durch Aufspaltung von _kb mit Trenner ':'
   */
  private String[] _kb_liste = {};
  /**
   * AnzeigeAktorTls mit Key AnzeigeAktorTls
   */
  private HashMap<String, AnzeigeAktorTls> m_listeAnzeigeAktorTls = new HashMap<String, AnzeigeAktorTls>();
  /**
   * AnzeigeAktorTls mit Key DEs
   */
  private HashMap<String, AnzeigeAktorTls> m_listeAnzeigeAktorTlsKeyDes = new HashMap<String, AnzeigeAktorTls>();
  /**
   * AnzeigeAktorTls mit Key Anzeige
   */
  private HashMap<String, AnzeigeAktorTls> m_listeAnzeigeAktorTlsKeyAnzeige = new HashMap<String, AnzeigeAktorTls>();
  /**
   * Attributgruppe atg.anzeigeEigenschaftIst für alle Anzeigen aktuell
   */
  private HashMap<String, Data> m_listeAnzeigeEigenschaftIst = new HashMap<String, Data>();
  /**
   * Anzeigestatus für alle Anzeigen aktuell
   */
  private HashMap<String, HashMap> m_listeAnzeigeStatus = new HashMap<String, HashMap>();
  /**
   * AnzeigeTypUmsetzungTls
   */
  private HashMap<String, AnzeigeTypUmsetzungTls> m_listeAnzeigeTypUmsetzungTls = new HashMap<String, AnzeigeTypUmsetzungTls>();
  /**
   * Liste mit Objekten für Typ Anzeige
   */
  private Vector<Anzeige> m_listeAnzeigeKlasse = new Vector<Anzeige>();
  /**
   * Liste mit DEs für AnzeigeAktorTls
   */
  private Vector<SystemObject> m_listeAnzeigeAktorTlsDes = new Vector<SystemObject>();
  /**
   * AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnittAktorTls
   */
  private HashMap<String, AnzeigeQuerschnittAktorTls> m_listeAnzeigeQuerschnittAktorTls = new HashMap<String, AnzeigeQuerschnittAktorTls>();
  /**
   * AnzeigeQuerschnittAktorTls mit Key DEs
   */
  private HashMap<String, AnzeigeQuerschnittAktorTls> m_listeAnzeigeQuerschnittAktorTlsKeyDes = new HashMap<String, AnzeigeQuerschnittAktorTls>();
  /**
   * AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt
   */
  private HashMap<String, AnzeigeQuerschnittAktorTls> m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt = new HashMap<String, AnzeigeQuerschnittAktorTls>();
  /**
   * Attributgruppe atg.anzeigeQuerschnittEigenschaftIst für alle AnzeigenQuerschnitte aktuell
   */
  private HashMap<String, Data> m_listeAnzeigeQuerschnittEigenschaftIst = new HashMap<String, Data>();
  /**
   * Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba für alle AnzeigenQuerschnitte aktuell
   */
  private HashMap<String, Data> m_listeAnzeigeQuerschnittSchaltbarkeitNba = new HashMap<String, Data>();
 /**
   * Querschnittstatus für alle AnzeigenQuerschnitt aktuell
   */
  private HashMap<String, HashMap> m_listeAnzeigeQuerschnittStatus = new HashMap<String, HashMap>();
  /**
   * AnzeigeQuerschnittTypUmsetzungTls
   */
  private HashMap<String, AnzeigeQuerschnittTypUmsetzungTls> m_listeAnzeigeQuerschnittTypUmsetzungTls = new HashMap<String, AnzeigeQuerschnittTypUmsetzungTls>();
  /**
   * Liste mit Objekten für Typ AnzeigeQuerschnitt
   */
  private Vector<SystemObject> m_listeAnzeigeQuerschnitt = new Vector<SystemObject>();
  /**
   * Liste mit Instanzen der Klasse AnzeigeQuerschnitt.
   */
  private Vector<AnzeigeQuerschnitt> m_listeAnzeigeQuerschnittKlasse = new Vector<AnzeigeQuerschnitt>();
  /**
   * Liste mit DEs für AnzeigeQuerschnittAktorTls
   */
  private Vector<SystemObject> m_listeAnzeigeQuerschnittAktorTlsDes = new Vector<SystemObject>();

  /**
   * DirigentUmsetzungTls
   */
  private HashMap<String, DirigentUmsetzungTls> m_listeDirigentUmsetzungTls = new HashMap<String, DirigentUmsetzungTls>();
  /**
   * DirigentAktorTls mit Key DE
   */
  private HashMap<String, DirigentAktorTls> m_listeDirigentAktorTlsKeyDes = new HashMap<String, DirigentAktorTls>();
  /**
   * DirigentAktorTls mit Key DirigentAktorTls
   */
  private HashMap<String, DirigentAktorTls> m_listeDirigentAktorTls = new HashMap<String, DirigentAktorTls>();
  /**
   * DirigentAktorTls mit Key Dirigent
   */
  private HashMap<String, DirigentAktorTls> m_listeDirigentAktorTlsKeyDirigent = new HashMap<String, DirigentAktorTls>();
  /**
   * Attributgruppe atg.dirigentEigenschaftIst für alle Dirigenten aktuell.
   */
  private HashMap<String, Data> m_listeDirigentEigenschaftIst = new HashMap<String, Data>();
  /**
   * Liste mit Objekten für Typ Dirigent.
   */
  private Vector<SystemObject> m_listeDirigent = new Vector<SystemObject>();
  /**
   * Liste mit Instanzen der Klasse Dirigent.
   */
  private Vector<Dirigent> m_listeDirigentKlasse = new Vector<Dirigent>();
  /**
   * Liste mit DEs für DirigentAktorTls
   */
  private Vector<SystemObject> m_listeDirigentAktorTlsDes = new Vector<SystemObject>();
  /**
   * Dirigentenstatus für alle Dirigenten aktuell
   */
  private HashMap<String, HashMap> m_listeDirigentStatus = new HashMap<String, HashMap>();

  /**
   * Index für den Zugriff auf die Attributgruppe in den Objekten davToZs... und zsToDav...
   */
  public static final int ATG = 0;
  /**
   * Index für den Zugriff auf den Aspekt in den Objekten davToZs... und zsToDav...
   */
  public static final int ASP = 1;
  /**
   * Instanz der Klasse KexInputAnzeige.
   */
  protected KexInputAnzeige m_kexInput;
  /**
   * AttributGruppe "atg.anzeigeQuerschnittNba".
   */
  protected AttributeGroup m_atgAnzeigeQuerschnittNba;

  /**
   * Konstruktor.
   */
  public ZwischenschichtAq()
  {
  }

  /**
   * Programmeinstieg.<br>
   *
   * @param args Kommandozeilenargumente.
   */
  public static void main(String[] args)
  {
    StandardApplicationRunner.run(new ZwischenschichtAq(), args);
  }

  /* (non-Javadoc)
   * @see sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    m_debug = Debug.getLogger();

    m_debug.fine("argumentList = " + argumentList);

    _kb = argumentList.fetchArgument("-bereiche=").asString();

    if (_kb.length() > 0)
      _kb_liste = _kb.split(":");

    for (int i = 0; i < _kb_liste.length; i++)
    {
      m_debug.info("Liste Bereiche " + i + " '" + _kb_liste[i] + "'");
    }
  }

  /* (non-Javadoc)
   * @see sys.funclib.application.StandardApplication#initialize(stauma.dav.clientside.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    try
    {
      m_connection = connection;

      m_dataModel = connection.getDataModel();

      m_atgAnzeigeQuerschnittNba = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittNba");

      m_debug.config("Connection " + m_connection + " DataModel " + m_dataModel);

      m_kexInput = new KexInputAnzeige(m_connection, this, getListeAnzeigeAktorTlsKeyDes(), getListeAnzeigeAktorTlsKeyAnzeige(), getListeAnzeigeEigenschaftIst(), getListeAnzeigeStatus());

      objekteZwischenschichtEinlesen(m_dataModel, m_kexInput);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Objekte der Zwischenschicht inklusive der ATGs aus der Konfiguration holen.
   *
   * @param dataModel Datenmodell.
   * @param kexInputAnzeige Instanz der Klasse KexInputAnzeige.
   */
  @SuppressWarnings("unchecked")
  public void objekteZwischenschichtEinlesen(DataModel dataModel, KexInputAnzeige kexInputAnzeige)
  {
    AttributeGroup atgAnzeigeAktor = dataModel.getAttributeGroup("atg.anzeigeAktor");
    AttributeGroup atgAnzeigeAktorTls = dataModel.getAttributeGroup("atg.anzeigeAktorTls");

//  Anzeige

//  alle Objekte vom Typ typ.anzeigeAktorTls aus der Konfiguration holen

    SystemObjectType typAnzeigeAktorTls = dataModel.getType("typ.anzeigeAktorTls");

    List tmpElemente = typAnzeigeAktorTls.getElements();

//  Liste der Objekte vom Typ typ.anzeigeAktorTls reduzieren auf Objekte in übergebenen Konfigurationsbereichen

    List<SystemObject> elementeAnzeigeAktorTls = new Vector<SystemObject>();

    for (int i = 0; i < tmpElemente.size(); i++)
    {
      SystemObject anzeigeAktorTls = (SystemObject)tmpElemente.get(i);

      if (!pruefeKb(anzeigeAktorTls)) // nur Objekte der übergebenen Konfigurationsbereiche verwenden
      {
        continue;
      }

      elementeAnzeigeAktorTls.add(anzeigeAktorTls);
    }

//  Attributdaten der Objekte vom Typ typ.anzeigeAktorTls für schnellen Zugriff in DAF holen

    m_dataModel.getConfigurationData(elementeAnzeigeAktorTls, atgAnzeigeAktor);
    m_dataModel.getConfigurationData(elementeAnzeigeAktorTls, atgAnzeigeAktorTls);

//  Objekte für Umsetzung erstellen

    for (int i = 0; i < elementeAnzeigeAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeAktorTls " + i + " " + elementeAnzeigeAktorTls.get(i));

      ConfigurationObject anzeigeAktorTls = (ConfigurationObject)elementeAnzeigeAktorTls.get(i);

      AnzeigeAktorTls aktorTls = new AnzeigeAktorTls(anzeigeAktorTls);

      Data datenAtgAnzeigeAktor = anzeigeAktorTls.getConfigurationData(atgAnzeigeAktor);

      String anzeigeReferenz = datenAtgAnzeigeAktor.getReferenceValue("AnzeigeReferenz").getValueText();

      aktorTls.setAnzeige(m_dataModel.getObject(anzeigeReferenz));

      m_listeAnzeigeEigenschaftIst.put(aktorTls.getAnzeige().getPid(), atgAnzeigeEigenschaftIstErstellen());

      String anzeigeTypUmsetzungReferenz = datenAtgAnzeigeAktor.getReferenceValue("AnzeigeTypUmsetzungReferenz").getValueText();

      aktorTls.setAnzeigeTypUmsetzung(anzeigeTypUmsetzungHolenBzwErstellen(anzeigeTypUmsetzungReferenz));

      Data datenAtgAnzeigeAktorTls = anzeigeAktorTls.getConfigurationData(atgAnzeigeAktorTls);

      int anzahl = datenAtgAnzeigeAktorTls.getReferenceArray("ZugehörigeDEs").getLength();

      SystemObject[] system_objekte = new SystemObject[anzahl];

      for (int j = 0; j < anzahl; j++)
      {
        ReferenceValue referenz = datenAtgAnzeigeAktorTls.getReferenceArray("ZugehörigeDEs").getReferenceValue(j);

        m_debug.config("Referenz " + j + " " + referenz);

        system_objekte[j] = dataModel.getObject(referenz.getValueText());

        aktorTls.getDes().put(system_objekte[j].getPid(), system_objekte[j]);
      }

      if (aktorTls.getDes().size() > 0)
      {
        SystemObject de = aktorTls.getDes().get(0);

        SystemObjectType typEak = m_dataModel.getType("typ.eak");

        List tmpEak = typEak.getElements();

        for (int j = 0; j < tmpEak.size(); j++)
        {
          ConfigurationObject eak = (ConfigurationObject)tmpEak.get(j);

          List tmpDe = eak.getNonMutableSet("De").getElements();

          for (int k = 0; k < tmpDe.size(); k++)
          {
            ConfigurationObject deTmp = (ConfigurationObject)tmpDe.get(k);

            if (deTmp.equals(de))
            {
              aktorTls.setEak(eak);

              break;
            }
          }
        }
      }

      m_listeAnzeigeAktorTls.put(aktorTls.getPid(), aktorTls);
      m_listeAnzeigeAktorTlsKeyAnzeige.put(aktorTls.getAnzeige().getPid(), aktorTls);

      Set liste = aktorTls.getDes().keySet();
      Iterator iterator = liste.iterator();

      while (iterator.hasNext())
      {
        m_listeAnzeigeAktorTlsKeyDes.put((String)iterator.next(), aktorTls);
      }
    }

    AttributeGroup atgAnzeigeAktor1 = m_dataModel.getAttributeGroup("atg.anzeigeAktor");

    //  Anzeigen für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeAnzeigeAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeAktorTls " + i + " " + elementeAnzeigeAktorTls.get(i));

      ConfigurationObject anzeigeAktorTls = (ConfigurationObject)elementeAnzeigeAktorTls.get(i);

      Data datenAtgAnzeigeAktorTls = anzeigeAktorTls.getConfigurationData(atgAnzeigeAktor1);

      ReferenceValue referenz = datenAtgAnzeigeAktorTls.getReferenceValue("AnzeigeReferenz");

//      m_debug.error("Anzeige " + referenz.getValueText());

      if (referenz != null)
      {
        ConfigurationObject anzeigeObjekt;

        anzeigeObjekt = (ConfigurationObject)m_dataModel.getObject(referenz.getValueText());

        m_listeAnzeigeKlasse.add(new Anzeige(m_connection, anzeigeObjekt, getListeAnzeigeAktorTlsKeyDes(), getListeAnzeigeAktorTlsKeyAnzeige()));
      }
    }

//  AnzeigeQuerschnitt

    AttributeGroup atgAnzeigeQuerschnittAktor = dataModel.getAttributeGroup("atg.anzeigeQuerschnittAktor");
    AttributeGroup atgAnzeigeQuerschnittAktorTls = dataModel.getAttributeGroup("atg.anzeigeQuerschnittAktorTls");

//  alle Objekte vom Typ typ.anzeigeQuerschnittAktorTls aus der Konfiguration holen

    SystemObjectType typAnzeigeQuerschnittAktorTls = dataModel.getType("typ.anzeigeQuerschnittAktorTls");

    tmpElemente = typAnzeigeQuerschnittAktorTls.getElements();

//  Liste der Objekte vom Typ typ.anzeigeQuerschnittAktorTls reduzieren auf Objekte in übergebenen Konfigurationsbereichen

    List<SystemObject> elementeAnzeigeQuerschnittAktorTls = new Vector<SystemObject>();

    for (int i = 0; i < tmpElemente.size(); i++)
    {
      SystemObject anzeigeQuerschnittAktorTls = (SystemObject)tmpElemente.get(i);

      if (!pruefeKb(anzeigeQuerschnittAktorTls)) // nur Objekte der übergebenen Konfigurationsbereiche verwenden
      {
        continue;
      }

      elementeAnzeigeQuerschnittAktorTls.add(anzeigeQuerschnittAktorTls);
    }

//  Attributdaten der Objekte vom Typ typ.anzeigeQuerschnittAktorTls für schnellen Zugriff in DAF holen

    m_dataModel.getConfigurationData(elementeAnzeigeQuerschnittAktorTls, atgAnzeigeQuerschnittAktor);
    m_dataModel.getConfigurationData(elementeAnzeigeQuerschnittAktorTls, atgAnzeigeQuerschnittAktorTls);

//  Objekte für Umsetzung erstellen

    for (int i = 0; i < elementeAnzeigeQuerschnittAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeQuerschnittAktorTls " + i + " " + elementeAnzeigeQuerschnittAktorTls.get(i));

      ConfigurationObject anzeigeQuerschnittAktorTls = (ConfigurationObject)elementeAnzeigeQuerschnittAktorTls.get(i);

      AnzeigeQuerschnittAktorTls aktorTls = new AnzeigeQuerschnittAktorTls(anzeigeQuerschnittAktorTls);

      Data datenAtgAnzeigeQuerschnittAktor = anzeigeQuerschnittAktorTls.getConfigurationData(atgAnzeigeQuerschnittAktor);

      String anzeigeReferenz = datenAtgAnzeigeQuerschnittAktor.getReferenceValue("AnzeigeQuerschnittReferenz").getValueText();

      aktorTls.setAnzeigeQuerschnitt(m_dataModel.getObject(anzeigeReferenz));

      m_listeAnzeigeQuerschnittEigenschaftIst.put(aktorTls.getAnzeigeQuerschnitt().getPid(), atgAnzeigeQuerschnittEigenschaftIstErstellen());

      ConfigurationObject anzeigeQuerschnitt = (ConfigurationObject)aktorTls.getAnzeigeQuerschnitt();
      Data dataAnzeigeQuerschnittNba = anzeigeQuerschnitt.getConfigurationData(m_atgAnzeigeQuerschnittNba);

      boolean immerSchaltbar = true;

      if (dataAnzeigeQuerschnittNba != null)
      {
        immerSchaltbar = !dataAnzeigeQuerschnittNba.getScaledValue("VorgelagerterEntscheiderVorhanden").getValueText().equals("Ja");
      }

      aktorTls.setImmerSchaltbar(immerSchaltbar);

      m_listeAnzeigeQuerschnittSchaltbarkeitNba.put(aktorTls.getAnzeigeQuerschnitt().getPid(), atgAnzeigeQuerschnittSchaltbarkeitNbaErstellen(immerSchaltbar));

      String anzeigeQuerschnittTypUmsetzungReferenz = datenAtgAnzeigeQuerschnittAktor.getReferenceValue("AnzeigeQuerschnittTypUmsetzungReferenz").getValueText();

      aktorTls.setAnzeigeQuerschnittTypUmsetzung(anzeigeQuerschnittTypUmsetzungHolenBzwErstellen(anzeigeQuerschnittTypUmsetzungReferenz));

      Data datenAtgAnzeigeQuerschnittAktorTls = anzeigeQuerschnittAktorTls.getConfigurationData(atgAnzeigeQuerschnittAktorTls);

      ReferenceValue referenz = datenAtgAnzeigeQuerschnittAktorTls.getReferenceValue("ZugehörigeDE");

      if (referenz != null)
      {
        SystemObject system_objekt = dataModel.getObject(referenz.getValueText());

        aktorTls.setDe(system_objekt);
      }

      if (aktorTls.getDe() != null)
      {
        SystemObject de = aktorTls.getDe();

        SystemObjectType typEak = m_dataModel.getType("typ.eak");

        List tmpEak = typEak.getElements();

        for (int j = 0; j < tmpEak.size(); j++)
        {
          ConfigurationObject eak = (ConfigurationObject)tmpEak.get(j);

          List tmpDe = eak.getNonMutableSet("De").getElements();

          for (int k = 0; k < tmpDe.size(); k++)
          {
            ConfigurationObject deTmp = (ConfigurationObject)tmpDe.get(k);

            if (deTmp.equals(de))
            {
              aktorTls.setEak(eak);

              break;
            }
          }
        }
      }

      m_listeAnzeigeQuerschnittAktorTls.put(aktorTls.getPid(), aktorTls);
      m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt.put(aktorTls.getAnzeigeQuerschnitt().getPid(), aktorTls);
      m_listeAnzeigeQuerschnittAktorTlsKeyDes.put(aktorTls.getDe().getPid(), aktorTls);
    }

    AttributeGroup atgAnzeigeQuerschnittAktor1 = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittAktor");

    //  Anzeigen für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeAnzeigeQuerschnittAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeQuerschnittAktorTls " + i + " " + elementeAnzeigeQuerschnittAktorTls.get(i));

      ConfigurationObject anzeigeQuerschnittAktorTls = (ConfigurationObject)elementeAnzeigeQuerschnittAktorTls.get(i);

      Data datenAtgAnzeigeQuerschnittAktorTls = anzeigeQuerschnittAktorTls.getConfigurationData(atgAnzeigeQuerschnittAktor1);

      ReferenceValue referenz = datenAtgAnzeigeQuerschnittAktorTls.getReferenceValue("AnzeigeQuerschnittReferenz");

//      m_debug.error("AnzeigeQuerschnitt " + referenz.getValueText());

      if (referenz != null)
      {
        ConfigurationObject anzeigeQuerschnittObjekt;

        anzeigeQuerschnittObjekt = (ConfigurationObject)m_dataModel.getObject(referenz.getValueText());

        m_listeAnzeigeQuerschnittKlasse.add(new AnzeigeQuerschnitt(m_connection, this, anzeigeQuerschnittObjekt, m_listeAnzeigeKlasse, m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt));
      }
    }

/*
    for (int i = 0; i < m_listeAnzeigeKlasse.size(); i++)
    {
      Anzeige anzeigeTmp = m_listeAnzeigeKlasse.get(i);

      System.out.println("Anzeige " + anzeigeTmp.getPid() + " AnzeigeQuerschnitt " + anzeigeTmp.getAnzeigeQuerschnitt().getPid());
    }
*/






//  Dirigenten

    AttributeGroup atgDirigentAktor = dataModel.getAttributeGroup("atg.dirigentAktor");
    AttributeGroup atgDirigentAktorTls = dataModel.getAttributeGroup("atg.dirigentAktorTls");

//  alle Objekte vom Typ typ.dirigentAktorTls aus der Konfiguration holen

    SystemObjectType typDirigentAktorTls = dataModel.getType("typ.dirigentAktorTls");

    tmpElemente = typDirigentAktorTls.getElements();

//  Liste der Objekte vom Typ typ.anzeigeQuerschnittAktorTls reduzieren auf Objekte in bergebenen Konfigurationsbereichen

    List<SystemObject> elementeDirigentAktorTls = new Vector<SystemObject>();

    for (int i = 0; i < tmpElemente.size(); i++)
    {
      SystemObject dirigentAktorTls = (SystemObject)tmpElemente.get(i);

      if (!pruefeKb(dirigentAktorTls)) // nur Objekte der übergebenen Konfigurationsbereiche verwenden
      {
        continue;
      }

      elementeDirigentAktorTls.add(dirigentAktorTls);
    }

//  Attributdaten der Objekte vom Typ typ.dirigentAktorTls für schnellen Zugriff in DAF holen

    m_dataModel.getConfigurationData(elementeDirigentAktorTls, atgDirigentAktor);
    m_dataModel.getConfigurationData(elementeDirigentAktorTls, atgDirigentAktorTls);

//  Objekte für Umsetzung erstellen

    for (int i = 0; i < elementeDirigentAktorTls.size(); i++)
    {
      m_debug.config("DirigentAktorTls " + i + " " + elementeDirigentAktorTls.get(i));

      ConfigurationObject dirigentAktorTls = (ConfigurationObject)elementeDirigentAktorTls.get(i);

      DirigentAktorTls aktorTls = new DirigentAktorTls(dirigentAktorTls);

      Data datenAtgDirigentAktor = dirigentAktorTls.getConfigurationData(atgDirigentAktor);

      String dirigentReferenz = datenAtgDirigentAktor.getReferenceValue("DirigentReferenz").getValueText();

      aktorTls.setDirigent(m_dataModel.getObject(dirigentReferenz));

      m_listeDirigentEigenschaftIst.put(aktorTls.getDirigent().getPid(), atgDirigentEigenschaftIstErstellen());

      ConfigurationObject dirigent = (ConfigurationObject)aktorTls.getDirigent();

      String dirigentUmsetzungReferenz = datenAtgDirigentAktor.getReferenceValue("DirigentUmsetzungReferenz").getValueText();

      aktorTls.setDirigentUmsetzung(dirigentUmsetzungHolenBzwErstellen(dirigentUmsetzungReferenz));

      Data datenAtgDirigentAktorTls = dirigentAktorTls.getConfigurationData(atgDirigentAktorTls);

      ReferenceValue referenz = datenAtgDirigentAktorTls.getReferenceValue("ZugehörigeDE");

      if (referenz != null)
      {
        SystemObject system_objekt = dataModel.getObject(referenz.getValueText());

        aktorTls.setDe(system_objekt);
      }

      if (aktorTls.getDe() != null)
      {
        SystemObject de = aktorTls.getDe();

        SystemObjectType typEak = m_dataModel.getType("typ.eak");

        List tmpEak = typEak.getElements();

        for (int j = 0; j < tmpEak.size(); j++)
        {
          ConfigurationObject eak = (ConfigurationObject)tmpEak.get(j);

          List tmpDe = eak.getNonMutableSet("De").getElements();

          for (int k = 0; k < tmpDe.size(); k++)
          {
            ConfigurationObject deTmp = (ConfigurationObject)tmpDe.get(k);

            if (deTmp.equals(de))
            {
              aktorTls.setEak(eak);

              break;
            }
          }
        }
      }

      m_listeDirigentAktorTls.put(aktorTls.getPid(), aktorTls);
      m_listeDirigentAktorTlsKeyDirigent.put(aktorTls.getDirigent().getPid(), aktorTls);
      m_listeDirigentAktorTlsKeyDes.put(aktorTls.getDe().getPid(), aktorTls);
    }

    AttributeGroup atgDirigentAktor1 = m_dataModel.getAttributeGroup("atg.dirigentAktor");

    //  Anzeigen für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeDirigentAktorTls.size(); i++)
    {
      m_debug.config("DirigentAktorTls " + i + " " + elementeDirigentAktorTls.get(i));

      ConfigurationObject dirigentAktorTls = (ConfigurationObject)elementeDirigentAktorTls.get(i);

      Data datenAtgDirigentAktorTls = dirigentAktorTls.getConfigurationData(atgDirigentAktor1);

      ReferenceValue referenz = datenAtgDirigentAktorTls.getReferenceValue("DirigentReferenz");

//      m_debug.error("Dirigent " + referenz.getValueText());

      if (referenz != null)
      {
        ConfigurationObject dirigentObjekt;

        dirigentObjekt = (ConfigurationObject)m_dataModel.getObject(referenz.getValueText());

        m_listeDirigentKlasse.add(new Dirigent(m_connection, this, dirigentObjekt, m_listeAnzeigeQuerschnittKlasse, m_listeDirigentAktorTlsKeyDirigent));
      }
    }


    for (int i = 0; i < m_listeDirigentKlasse.size(); i++)
    {
      Dirigent dirigentTmp = m_listeDirigentKlasse.get(i);

      System.out.println("Dirigent " + dirigentTmp.getPid());
    }


















    // Sender anmelden (immer vor Empfänger)

    for (int i = 0; i < m_listeAnzeigeKlasse.size(); i++)
    {
      m_listeAnzeigeKlasse.get(i).anmeldungDatenversand();
    }

    for (int i = 0; i < m_listeAnzeigeQuerschnittKlasse.size(); i++)
    {
      m_listeAnzeigeQuerschnittKlasse.get(i).anmeldungDatenversand();
    }

    for (int i = 0; i < m_listeDirigentKlasse.size(); i++)
    {
      m_listeDirigentKlasse.get(i).anmeldungDatenversand();
    }

    datenAnmeldungAnzeigeAktorTls(elementeAnzeigeAktorTls, kexInputAnzeige);

    datenAnmeldungAnzeigeQuerschnittAktorTls(elementeAnzeigeQuerschnittAktorTls, kexInputAnzeige);

    datenAnmeldungDirigentAktorTls(elementeDirigentAktorTls, kexInputAnzeige);

    // Empfänger anmelden

    for (int i = 0; i < m_listeAnzeigeKlasse.size(); i++)
    {
      m_listeAnzeigeKlasse.get(i).anmeldungDatenempfang();
    }

    for (int i = 0; i < m_listeAnzeigeQuerschnittKlasse.size(); i++)
    {
      m_listeAnzeigeQuerschnittKlasse.get(i).anmeldungDatenempfang();
    }

    for (int i = 0; i < m_listeDirigentKlasse.size(); i++)
    {
      m_listeDirigentKlasse.get(i).anmeldungDatenempfang();
    }
  }

  /**
   * Anmeldung der Daten für Objekte vom Typ DeWzg beim DAV.<br>
   * Empfang:<br>
   * (- atg.anzeigeEigenschaftSoll:asp.aqVorgabe<br>)
   *
   * Versand:<br>
   * (- atg.anzeigeEigenschaftIst:asp.aqZustand<br>)
   *
   * @param elementeAnzeigeAktorTls Liste der zu bearbeitenden Objekte vom Typ typ.anzeigeAktorTls.
   * @param kexInputAnzeige Instanz der Klasse KexInputAnzeige.
   */
  private void datenAnmeldungAnzeigeAktorTls(List<SystemObject> elementeAnzeigeAktorTls, KexInputAnzeige kexInputAnzeige)
  {
    String ausgabe;
    Aspect aspParameter;
    AttributeGroup atgParameter;
    DataDescription dd;
    AttributeGroup atgAnzeigeAktorTls = m_dataModel.getAttributeGroup("atg.anzeigeAktorTls");

    //  AnzeigeAktorTls-DEs für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeAnzeigeAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeAktorTls " + i + " " + elementeAnzeigeAktorTls.get(i));

      ConfigurationObject anzeigeAktorTls = (ConfigurationObject)elementeAnzeigeAktorTls.get(i);

      Data datenAtgAnzeigeAktorTls = anzeigeAktorTls.getConfigurationData(atgAnzeigeAktorTls);

      int anzahl = datenAtgAnzeigeAktorTls.getReferenceArray("ZugehörigeDEs").getLength();

      SystemObject deObjekt;

      for (int j = 0; j < anzahl; j++)
      {
        ReferenceValue referenz = datenAtgAnzeigeAktorTls.getReferenceArray("ZugehörigeDEs").getReferenceValue(j);

        m_debug.config("Referenz " + j + " " + referenz);

        deObjekt = m_dataModel.getObject(referenz.getValueText());

        m_listeAnzeigeAktorTlsDes.add(deObjekt);
      }
    }

    ausgabe = "Anzumeldende DEs";

    for (int i = 0; i < m_listeAnzeigeAktorTlsDes.size(); i++)
    {
      ausgabe += "\nDE " + ((SystemObject)m_listeAnzeigeAktorTlsDes.get(i)).getPid();
    }

    m_debug.config(ausgabe);

//  AnzeigeAktorTls-DEs auf Daten von DEs anmelden

    for (int i = 0; i < m_listeAnzeigeAktorTlsDes.size(); i++)
    {
      SystemObject object = (SystemObject)m_listeAnzeigeAktorTlsDes.get(i);

      AnzeigeAktorTls anzeigeAktorUms = (AnzeigeAktorTls)kexInputAnzeige.getListeAnzeigeAktorTlsDes().get(object.getPid());

      m_debug.config(object.getPid() + " Umsetzung " + anzeigeAktorUms.getAnzeigeTypUmsetzung().getPid());

      AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktorUms.getAnzeigeTypUmsetzung();

      int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

      for (int j = 0; j < anzahlGlobalUmsetzung; j++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

        for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
        {
          GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();

          String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
          String asp = guvaga.getAttributGruppenAspekte().get(k).getPidEmpfangAspekt();

          atgParameter = m_dataModel.getAttributeGroup(atg);
          aspParameter = m_dataModel.getAspect(asp);

          dd = new DataDescription(atgParameter, aspParameter);

          m_debug.config("Anmeldung Empfänger " + object.getPid() + " " + dd);

          try
          {
            m_connection.subscribeReceiver(kexInputAnzeige, object, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
          }
          catch (NullPointerException e)
          {
            e.printStackTrace();

            System.exit(0);
          }
        }
      }
    }

//  AnzeigeAktorTls-DEs auf Befehle für DEs anmelden

    for (int i = 0; i < m_listeAnzeigeAktorTlsDes.size(); i++)
    {
      SystemObject object = (SystemObject)m_listeAnzeigeAktorTlsDes.get(i);

      AnzeigeAktorTls anzeigeAktorUms = (AnzeigeAktorTls)kexInputAnzeige.getListeAnzeigeAktorTlsDes().get(object.getPid());

      m_debug.config(object.getPid() + " Umsetzung " + anzeigeAktorUms.getAnzeigeTypUmsetzung().getPid());

      AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktorUms.getAnzeigeTypUmsetzung();

      int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

      for (int j = 0; j < anzahlGlobalUmsetzung; j++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

        for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
        {
          GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();

          String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
          String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();

          if (asp.length() == 0)
          {
            continue;
          }

          atgParameter = m_dataModel.getAttributeGroup(atg);
          aspParameter = m_dataModel.getAspect(asp);

          dd = new DataDescription(atgParameter, aspParameter);

          m_debug.config("Anmeldung Sender " + object.getPid() + " " + dd);

          try
          {
            m_connection.subscribeSender(kexInputAnzeige, object, dd, SenderRole.sender());
          }
          catch (OneSubscriptionPerSendData e)
          {
            m_debug.error("Fehler bei der Senderanmeldung " + object.getPid() + " " + atg + ":" + asp + " : " + e.getMessage());

//            e.printStackTrace();
//
//            System.exit(0);
          }
        }
      }
    }
  }

  /**
   * Anmeldung der Daten für Objekte vom Typ DeWzg (Cluster) beim DAV.<br>
   * Empfang:<br>
   * (- atg.anzeigeQuerschnittEigenschaftSoll:asp.aqVorgabe<br>)
   *
   * Versand:<br>
   * (- atg.anzeigeQuerschnittEigenschaftIst:asp.aqZustand<br>)
   *
   * @param elementeAnzeigeQuerschnittAktorTls Liste der zu bearbeitenden Objekte vom Typ typ.anzeigeQuerschnittAktorTls.
   * @param kexInputAnzeige Instanz der Klasse KexInputAnzeige.
   */
  private void datenAnmeldungAnzeigeQuerschnittAktorTls(List<SystemObject> elementeAnzeigeQuerschnittAktorTls, KexInputAnzeige kexInputAnzeige)
  {
    String ausgabe;
    Aspect aspParameter;
    AttributeGroup atgParameter;
    DataDescription dd;
    AttributeGroup atgAnzeigeQuerschnittAktor = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittAktor");
    AttributeGroup atgAnzeigeQuerschnittAktorTls = m_dataModel.getAttributeGroup("atg.anzeigeQuerschnittAktorTls");

//  AnzeigeQuerschnitt für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeAnzeigeQuerschnittAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeQuerschnittAktorTls " + i + " " + elementeAnzeigeQuerschnittAktorTls.get(i));

      ConfigurationObject anzeigeQuerschnittAktorTls = (ConfigurationObject)elementeAnzeigeQuerschnittAktorTls.get(i);

      Data datenAtgAnzeigeQuerschnittAktorTls = anzeigeQuerschnittAktorTls.getConfigurationData(atgAnzeigeQuerschnittAktor);

      ReferenceValue referenz = datenAtgAnzeigeQuerschnittAktorTls.getReferenceValue("AnzeigeQuerschnittReferenz");

      if (referenz != null)
      {
        SystemObject deObjekt;

        deObjekt = m_dataModel.getObject(referenz.getValueText());

        m_listeAnzeigeQuerschnitt.add(deObjekt);
      }
    }

    ausgabe = "";

    ausgabe += "Anzumeldende AnzeigeQuerschnitte (" + m_listeAnzeigeQuerschnitt.size() + ")";

    for (int i = 0; i < m_listeAnzeigeQuerschnitt.size(); i++)
    {
      ausgabe += "\nAnzeigeQuerschnitt " + ((SystemObject)m_listeAnzeigeQuerschnitt.get(i)).getPid();
    }

    m_debug.fine(ausgabe);

//  AnzeigeQuerschnittAktorTls-DEs für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeAnzeigeQuerschnittAktorTls.size(); i++)
    {
      m_debug.config("AnzeigeQuerschnittAktorTls " + i + " " + elementeAnzeigeQuerschnittAktorTls.get(i));

      ConfigurationObject anzeigeQuerschnittAktorTls = (ConfigurationObject)elementeAnzeigeQuerschnittAktorTls.get(i);

      Data datenAtgAnzeigeQuerschnittAktorTls = anzeigeQuerschnittAktorTls.getConfigurationData(atgAnzeigeQuerschnittAktorTls);

      ReferenceValue referenz = datenAtgAnzeigeQuerschnittAktorTls.getReferenceValue("ZugehörigeDE");

      if (referenz != null)
      {
        SystemObject deObjekt;

        deObjekt = m_dataModel.getObject(referenz.getValueText());

        m_listeAnzeigeQuerschnittAktorTlsDes.add(deObjekt);
      }
    }

    ausgabe = "";

    ausgabe += "Anzumeldende DEs";

    for (int i = 0; i < m_listeAnzeigeQuerschnittAktorTlsDes.size(); i++)
    {
      ausgabe += "\nDE " + ((SystemObject)m_listeAnzeigeQuerschnittAktorTlsDes.get(i)).getPid();
    }

    m_debug.fine(ausgabe);

//  AnzeigeQuerschnittAktorTls-DEs auf Daten von DEs anmelden

    for (int i = 0; i < m_listeAnzeigeQuerschnittAktorTlsDes.size(); i++)
    {
      SystemObject object = (SystemObject)m_listeAnzeigeQuerschnittAktorTlsDes.get(i);

      AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktorUms = (AnzeigeQuerschnittAktorTls)kexInputAnzeige.getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

      m_debug.config(object.getPid() + " Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid());

      AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();

      int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeGlobalUmsetzung().size();

      for (int j = 0; j < anzahlGlobalUmsetzung; j++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

        for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
        {
          GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();

          String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
          String asp = guvaga.getAttributGruppenAspekte().get(k).getPidEmpfangAspekt();

          atgParameter = m_dataModel.getAttributeGroup(atg);
          aspParameter = m_dataModel.getAspect(asp);

          dd = new DataDescription(atgParameter, aspParameter);

          m_debug.config("Anmeldung Empfänger " + object.getPid() + " " + dd);

          try
          {
            m_connection.subscribeReceiver(kexInputAnzeige, object, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
          }
          catch (NullPointerException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();

            System.exit(0);
          }
        }
      }
    }

//  AnzeigeQuerschnittAktorTls-DEs auf Befehle für DEs anmelden

    for (int i = 0; i < m_listeAnzeigeQuerschnittAktorTlsDes.size(); i++)
    {
      SystemObject object = (SystemObject)m_listeAnzeigeQuerschnittAktorTlsDes.get(i);

      AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktorUms = (AnzeigeQuerschnittAktorTls)kexInputAnzeige.getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

      m_debug.config(object.getPid() + " Umsetzung " + anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung().getPid());

      AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungTls = anzeigeQuerschnittAktorUms.getAnzeigeQuerschnittTypUmsetzung();

      int anzahlGlobalUmsetzung = anzeigeQuerschnittTypUmsetzungTls.getListeGlobalUmsetzung().size();

      for (int j = 0; j < anzahlGlobalUmsetzung; j++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeQuerschnittTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

        for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
        {
          GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();

          String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
          String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();

          if (asp.length() == 0)
          {
            continue;
          }

          atgParameter = m_dataModel.getAttributeGroup(atg);
          aspParameter = m_dataModel.getAspect(asp);

          dd = new DataDescription(atgParameter, aspParameter);

          m_debug.config("Anmeldung Sender " + object.getPid() + " " + dd);

          try
          {
            m_connection.subscribeSender(kexInputAnzeige, object, dd, SenderRole.sender());
          }
          catch (OneSubscriptionPerSendData e)
          {
            m_debug.error("Fehler bei der Senderanmeldung " + object.getPid() + " " + atg + ":" + asp + " : " + e.getMessage());

//            e.printStackTrace();

//            System.exit(0);
          }
        }
      }
    }
  }

  /**
   * Anmeldung der Daten für Objekte vom Typ DeWzg beim DAV.<br>
   * Empfang:<br>
   * (- atg.anzeigeEigenschaftSoll:asp.aqVorgabe<br>)
   *
   * Versand:<br>
   * (- atg.anzeigeEigenschaftIst:asp.aqZustand<br>)
   *
   * @param elementeDirigentAktorTls Liste der zu bearbeitenden Objekte vom Typ typ.dirigentAktorTls.
   * @param kexInputAnzeige Instanz der Klasse KexInputAnzeige.
   */
  private void datenAnmeldungDirigentAktorTls(List<SystemObject> elementeDirigentAktorTls, KexInputAnzeige kexInputAnzeige)
  {
    String ausgabe;
    Aspect aspParameter;
    AttributeGroup atgParameter;
    DataDescription dd;
    AttributeGroup atgDirigentAktorTls = m_dataModel.getAttributeGroup("atg.dirigentAktorTls");

    //  AnzeigeAktorTls-DEs für Anmeldung der Daten bestimmen

    for (int i = 0; i < elementeDirigentAktorTls.size(); i++)
    {
      m_debug.config("DirigentAktorTls " + i + " " + elementeDirigentAktorTls.get(i));

      ConfigurationObject dirigentAktorTls = (ConfigurationObject)elementeDirigentAktorTls.get(i);

      Data datenAtgDirigentAktorTls = dirigentAktorTls.getConfigurationData(atgDirigentAktorTls);

      ReferenceValue referenz = datenAtgDirigentAktorTls.getReferenceValue("ZugehörigeDE");

      if (referenz != null)
      {
        SystemObject deObjekt;

        deObjekt = m_dataModel.getObject(referenz.getValueText());

        m_listeDirigentAktorTlsDes.add(deObjekt);
      }
    }

    ausgabe = "Anzumeldende DEs";

    for (int i = 0; i < m_listeDirigentAktorTlsDes.size(); i++)
    {
      ausgabe += "\nDE " + ((SystemObject)m_listeDirigentAktorTlsDes.get(i)).getPid();
    }

    m_debug.config(ausgabe);

//  AnzeigeAktorTls-DEs auf Daten von DEs anmelden

    for (int i = 0; i < m_listeDirigentAktorTlsDes.size(); i++)
    {
      SystemObject object = (SystemObject)m_listeDirigentAktorTlsDes.get(i);

      DirigentAktorTls dirigentAktorUms = (DirigentAktorTls)kexInputAnzeige.getListeDirigentAktorTlsDes().get(object.getPid());

      m_debug.config(object.getPid() + " Umsetzung " + dirigentAktorUms.getDirigentUmsetzung().getPid());

      DirigentUmsetzungTls dirigentUmsetzungTls = dirigentAktorUms.getDirigentUmsetzung();

      int anzahlGlobalUmsetzung = dirigentUmsetzungTls.getListeGlobalUmsetzung().size();

      for (int j = 0; j < anzahlGlobalUmsetzung; j++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)dirigentUmsetzungTls.getListeGlobalUmsetzung().get(j);

        for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
        {
          GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();

          String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
          String asp = guvaga.getAttributGruppenAspekte().get(k).getPidEmpfangAspekt();

          atgParameter = m_dataModel.getAttributeGroup(atg);
          aspParameter = m_dataModel.getAspect(asp);

          dd = new DataDescription(atgParameter, aspParameter);

          m_debug.config("Anmeldung Empfänger " + object.getPid() + " " + dd);

          try
          {
            m_connection.subscribeReceiver(kexInputAnzeige, object, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
          }
          catch (NullPointerException e)
          {
            e.printStackTrace();

            System.exit(0);
          }
        }
      }
    }

//  DirigentAktorTls-DEs auf Befehle für DEs anmelden

    for (int i = 0; i < m_listeDirigentAktorTlsDes.size(); i++)
    {
      SystemObject object = (SystemObject)m_listeDirigentAktorTlsDes.get(i);

      DirigentAktorTls dirigentAktorUms = (DirigentAktorTls)kexInputAnzeige.getListeDirigentAktorTlsDes().get(object.getPid());

      m_debug.config(object.getPid() + " Umsetzung " + dirigentAktorUms.getDirigentUmsetzung().getPid());

      DirigentUmsetzungTls dirigentUmsetzungTls = dirigentAktorUms.getDirigentUmsetzung();

      int anzahlGlobalUmsetzung = dirigentUmsetzungTls.getListeGlobalUmsetzung().size();

      for (int j = 0; j < anzahlGlobalUmsetzung; j++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)dirigentUmsetzungTls.getListeGlobalUmsetzung().get(j);

        for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
        {
          GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();

          String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
          String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();

          if (asp.length() == 0)
          {
            continue;
          }

          atgParameter = m_dataModel.getAttributeGroup(atg);
          aspParameter = m_dataModel.getAspect(asp);

          dd = new DataDescription(atgParameter, aspParameter);

          m_debug.config("Anmeldung Sender " + object.getPid() + " " + dd);

          try
          {
            m_connection.subscribeSender(kexInputAnzeige, object, dd, SenderRole.sender());
          }
          catch (OneSubscriptionPerSendData e)
          {
            m_debug.error("Fehler bei der Senderanmeldung " + object.getPid() + " " + atg + ":" + asp + " : " + e.getMessage());

//            e.printStackTrace();
//
//            System.exit(0);
          }
        }
      }
    }
  }

  /* (non-Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
   */
  public void update(ResultData[] results)
  {
    try
    {
      for(int resultIndex = 0; resultIndex < results.length; ++resultIndex)
      {
        ResultData result = results[resultIndex];
//        SystemObject object = result.getObject();
//
//        AttributeGroup atg = result.getDataDescription().getAttributeGroup();
//        Aspect         asp = result.getDataDescription().getAspect();

        m_debug.config("Result " + result.toString());
      }
    }
    catch (Exception e)
    {
      // TODO: handle exception
      e.printStackTrace();
    }
  }

  /**
   * Holen der HashMap mit den AnzeigeAktorTls mit Key AnzeigeAktorTls.
   *
   * @return HashMap mit den AnzeigeAktorTls mit Key AnzeigeAktorTls.
   */
  public HashMap getListeAnzeigeAktorTls()
  {
    return m_listeAnzeigeAktorTls;
  }

  /**
   * Holen der HashMap mit den AnzeigeAktorTls mit Key Anzeige.
   *
   * @return HashMap mit den AnzeigeAktorTls mit Key Anzeige.
   */
  public HashMap getListeAnzeigeAktorTlsKeyAnzeige()
  {
    return m_listeAnzeigeAktorTlsKeyAnzeige;
  }

  /**
   * Holen der HashMap mit den AnzeigeAktorTls mit Key DEs.
   *
   * @return HashMap mit den AnzeigeAktorTls mit Key DEs.
   */
  public HashMap getListeAnzeigeAktorTlsKeyDes()
  {
    return m_listeAnzeigeAktorTlsKeyDes;
  }

  /**
   * Holen der HashMap mit den Attributgruppen atg.anzeigeEigenschaftIst für alle Anzeigen aktuell.
   *
   * @return HashMap mit den Attributgruppen atg.anzeigeEigenschaftIst für alle Anzeigen aktuell.
   */
  public HashMap getListeAnzeigeEigenschaftIst()
  {
    return m_listeAnzeigeEigenschaftIst;
  }

  /**
   * Holen der HashMap mit den Anzeigestatus für alle Anzeigen aktuell.
   *
   * @return HashMap mit den Anzeigestatus für alle Anzeigen aktuell.
   */
  public HashMap<String, HashMap> getListeAnzeigeStatus()
  {
    return m_listeAnzeigeStatus;
  }

  /**
   * Erzeugen der Initialen ATG atg.anzeigeEigenschaftIst.
   *
   * @return Datensatz mit der initialisierten ATG atg.anzeigeEigenschaftIst
   */
  private Data atgAnzeigeEigenschaftIstErstellen()
  {
    String atgPid = "atg.anzeigeEigenschaftIst";

    Data atgData = m_connection.createData(m_dataModel.getAttributeGroup(atgPid));

    atgAnzeigeEigenschaftIstInitialisieren(atgData, m_connection.getTime(), "Undefiniert");

    return atgData;
  }

  /**
   * Erzeugen der Initialen ATG atg.dirigentEigenschaftIst.
   *
   * @return Datensatz mit der initialisierten ATG atg.dirigentEigenschaftIst
   */
  private Data atgDirigentEigenschaftIstErstellen()
  {
    String atgPid = "atg.dirigentEigenschaftIst";

    Data atgData = m_connection.createData(m_dataModel.getAttributeGroup(atgPid));

    atgDirigentEigenschaftIstInitialisieren(atgData, m_connection.getTime(), "Undefiniert");

    return atgData;
  }

  /**
   * Initialisieren aller Attribute der Attributgruppe auf Initialwerte und Setzen der übergebenen Zeit und des übergebenen Status.
   *
   * @param data Attributgruppe.
   * @param time Zeit.
   * @param status Status.
   */
  static public void atgAnzeigeEigenschaftIstInitialisieren(Data data, long time, String status)
  {
    data.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
    data.getItem("Eigenschaft").getScaledValue("Blinken").setText("Nein");
    data.getItem("Eigenschaft").getScaledValue("Blinktakt").setText("Undefiniert");
    data.getItem("Eigenschaft").getScaledValue("Passiviert").setText("Nein");
    data.getItem("Eigenschaft").getReferenceArray("GrafikDarstellungen").setLength(0);
    data.getItem("Eigenschaft").getReferenceValue("Wechseltext").setSystemObject(null);
    data.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
    data.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
    data.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
    data.getScaledValue("Status").setText(status);
    data.getReferenceArray("GestörteInhalte").setLength(0);
    data.getArray("GestörteTextpositionen").setLength(0);
    data.getReferenceArray("AktFehlerMeldungen").setLength(0);
  }

  /**
   * Initialisieren aller Attribute der Attributgruppe auf Initialwerte und Setzen der übergebenen Zeit und des übergebenen Status.
   *
   * @param data Attributgruppe.
   * @param time Zeit.
   * @param status Status.
   */
  static public void atgDirigentEigenschaftIstInitialisieren(Data data, long time, String status)
  {
    data.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
    data.getItem("Eigenschaft").getScaledValue("Passiviert").setText("Nein");
    data.getScaledValue("Status").setText(status);
    data.getScaledValue("FehlerAufgetreten").setText("Nein");
    data.getScaledValue("ProgrammAbgeschlossen").setText("Ja");
  }

  /**
   * Holen bzw. Erstellen der AnzeigeTypUmsetzungTls mit der übergebenen Pid.
   *
   * @param pid Pid der AnzeigeTypUmsetzungTls.
   * @return AnzeigeTypUmsetzungTls.
   */
  private AnzeigeTypUmsetzungTls anzeigeTypUmsetzungHolenBzwErstellen(String pid)
  {
    AnzeigeTypUmsetzungTls anzeigeTypUmsetzung = null;

    anzeigeTypUmsetzung = (AnzeigeTypUmsetzungTls)m_listeAnzeigeTypUmsetzungTls.get(pid);

    if (anzeigeTypUmsetzung == null)
    {
      SystemObject objekt = m_dataModel.getObject(pid);

      anzeigeTypUmsetzung = new AnzeigeTypUmsetzungTls(m_dataModel, pid, objekt);

      m_listeAnzeigeTypUmsetzungTls.put(pid, anzeigeTypUmsetzung);
    }

    return anzeigeTypUmsetzung;
  }

  /**
   * Diese Methode überprüft den Konfigurationsbereich des übergebenen Objekts.
   * Ist die Liste der Konfigurationsbereiche leer, wird 'true' zurückgegeben.
   * Ist die Liste der Konfigurationsbereiche nicht leer, so wird nur dann 'true'
   * zurückgegeben, wenn der Konfigurationsbereich des Objektes in der Liste
   * aufgeführt ist
   *
   * @param objekt zu prüfendes Objekt
   * @return true, wenn der Konfigurationsbereich des zu prüfendes Objekt in der Liste der Konfigurationsbereiche vorhanden oder die Liste leer ist
   */
  protected boolean pruefeKb(SystemObject objekt)
  {
    boolean ergebnis = false;

    if (_kb_liste.length == 0)
    {
      ergebnis = true;
    }
    else
    {
      String kb = objekt.getConfigurationArea().getPid();

      for (int i = 0; ((i < _kb_liste.length) && (!ergebnis)); i++)
      {
        ergebnis = kb.equals(_kb_liste[i]);
      }
    }

    return ergebnis;
  }

  /**
   * Erzeugen der Initialen ATG atg.anzeigeQuerschnittEigenschaftIst.
   *
   * @return Datensatz mit der initialisierten ATG atg.anzeigeQuerschnittEigenschaftIst
   */
  private Data atgAnzeigeQuerschnittEigenschaftIstErstellen()
  {
    String atgPid = "atg.anzeigeQuerschnittEigenschaftIst";

    Data atgData = m_connection.createData(m_dataModel.getAttributeGroup(atgPid));

    atgAnzeigeQuerschnittEigenschaftIstInitialisieren(atgData, m_connection.getTime(), "Undefiniert");

    return atgData;
  }

  /**
   * Erzeugen der Initialen ATG atg.anzeigeQuerschnittSchaltbarkeitNba.
   *
   * @param immerSchaltbar <code>true</code> zeigt an, dass kein vorgelagerter Entscheider vorhanden ist und daher immer geschaltet werden kann.
   * @return Datensatz mit der initialisierten ATG atg.anzeigeQuerschnittSchaltbarkeitNba.
   */
  private Data atgAnzeigeQuerschnittSchaltbarkeitNbaErstellen(boolean immerSchaltbar)
  {
    String atgPid = "atg.anzeigeQuerschnittSchaltbarkeitNba";

    Data atgData = m_connection.createData(m_dataModel.getAttributeGroup(atgPid));

    atgAnzeigeQuerschnittSchaltbarkeitNbaInitialisieren(atgData, immerSchaltbar);

    return atgData;
  }

  /**
   * Holen bzw. Erstellen der AnzeigeQuerschnittTypUmsetzungTls mit der übergebenen Pid.
   *
   * @param pid Pid der AnzeigeQuerschnittTypUmsetzungTls.
   * @return AnzeigeQuerschnittTypUmsetzungTls.
   */
  private AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzungHolenBzwErstellen(String pid)
  {
    AnzeigeQuerschnittTypUmsetzungTls anzeigeQuerschnittTypUmsetzung = null;

    anzeigeQuerschnittTypUmsetzung = (AnzeigeQuerschnittTypUmsetzungTls)m_listeAnzeigeQuerschnittTypUmsetzungTls.get(pid);

    if (anzeigeQuerschnittTypUmsetzung == null)
    {
      SystemObject objekt = m_dataModel.getObject(pid);

      anzeigeQuerschnittTypUmsetzung = new AnzeigeQuerschnittTypUmsetzungTls(m_dataModel, pid, objekt);

      m_listeAnzeigeQuerschnittTypUmsetzungTls.put(pid, anzeigeQuerschnittTypUmsetzung);
    }

    return anzeigeQuerschnittTypUmsetzung;
  }

  /**
   * Holen bzw. Erstellen der DirigentUmsetzungTls mit der übergebenen Pid.
   *
   * @param pid Pid der DirigentUmsetzungTls.
   * @return DirigentUmsetzungTls.
   */
  private DirigentUmsetzungTls dirigentUmsetzungHolenBzwErstellen(String pid)
  {
    DirigentUmsetzungTls dirigentUmsetzung = null;

    dirigentUmsetzung = (DirigentUmsetzungTls)m_listeDirigentUmsetzungTls.get(pid);

    if (dirigentUmsetzung == null)
    {
      SystemObject objekt = m_dataModel.getObject(pid);

      dirigentUmsetzung = new DirigentUmsetzungTls(m_dataModel, pid, objekt);

      m_listeDirigentUmsetzungTls.put(pid, dirigentUmsetzung);
    }

    return dirigentUmsetzung;
  }

  /**
   * Initialisieren aller Attribute der Attributgruppe auf Initialwerte und Setzen der übergebenen Zeit und des übergebenen Status.
   *
   * @param data Attributgruppe.
   * @param time Zeit.
   * @param status Status.
   */
  static public void atgAnzeigeQuerschnittEigenschaftIstInitialisieren(Data data, long time, String status)
  {
    data.getItem("Eigenschaft").getScaledValue("Helligkeit").setText("nicht ermittelbar");
    data.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText("Undefiniert");
    data.getItem("Eigenschaft").getScaledValue("Passiviert").setText("Nein");
    data.getReferenceArray("Anzeigen").setLength(0);
    data.getScaledValue("Status").setText(status);
    data.getReferenceArray("AktFehlerMeldungen").setLength(0);
  }

  /**
   * Initialisieren aller Attribute der Attributgruppe auf Initialwerte.<br>
   * Wenn 'immerSchaltbar' <code>true</code> wird das Attribut 'DurchNbaSchaltbar' auf 'Ja', ansonsten auf 'nicht ermittelbar' gesetzt.
   *
   * @param data zu initialisierender Datensatz.
   * @param immerSchaltbar <code>true</code> zeigt an, dass kein vorgelagerter Entscheider vorhanden ist und daher immer geschaltet werden kann.
   */
  static public void atgAnzeigeQuerschnittSchaltbarkeitNbaInitialisieren(Data data, boolean immerSchaltbar)
  {
    if (immerSchaltbar)
    {
      data.getScaledValue("DurchNbaSchaltbar").setText("Ja");
    }
    else
    {
      data.getScaledValue("DurchNbaSchaltbar").setText("nicht ermittelbar");
    }
  }

  /**
   * Holen der HashMap mit den AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnittAktorTls.
   *
   * @return HashMap mit den AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnittAktorTls.
   */
  public HashMap<String, AnzeigeQuerschnittAktorTls> getListeAnzeigeQuerschnittAktorTls()
  {
    return m_listeAnzeigeQuerschnittAktorTls;
  }

  /**
   * Holen der Liste mit DEs für AnzeigeQuerschnittAktorTls.
   *
   * @return Liste mit DEs für AnzeigeQuerschnittAktorTls.
   */
  public Vector<SystemObject> getListeAnzeigeQuerschnittAktorTlsDes()
  {
    return m_listeAnzeigeQuerschnittAktorTlsDes;
  }

  /**
   * Holen der HashMap mit den AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   *
   * @return HashMap mit den AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   */
  public HashMap<String, AnzeigeQuerschnittAktorTls> getListeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt()
  {
    return m_listeAnzeigeQuerschnittAktorTlsKeyAnzeigeQuerschnitt;
  }

  /**
   * Holen der HashMap mit den AnzeigeQuerschnittAktorTls mit Key DEs.
   *
   * @return HashMap mit den AnzeigeQuerschnittAktorTls mit Key DEs.
   */
  public HashMap<String, AnzeigeQuerschnittAktorTls> getListeAnzeigeQuerschnittAktorTlsKeyDes()
  {
    return m_listeAnzeigeQuerschnittAktorTlsKeyDes;
  }

  /**
   * Holen der HashMap mit der Attributgruppe atg.anzeigeQuerschnittEigenschaftIst für alle AnzeigenQuerschnitte aktuell.
   *
   * @return HashMap mit der Attributgruppe atg.anzeigeQuerschnittEigenschaftIst für alle AnzeigenQuerschnitte aktuell.
   */
  public HashMap<String, Data> getListeAnzeigeQuerschnittEigenschaftIst()
  {
    return m_listeAnzeigeQuerschnittEigenschaftIst;
  }

  /**
   * Holen der HashMap mit der Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba für alle AnzeigenQuerschnitte aktuell.
   *
   * @return HashMap mit der Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba für alle AnzeigenQuerschnitte aktuell.
   */
  public HashMap<String, Data> getListeAnzeigeQuerschnittSchaltbarkeitNba()
  {
    return m_listeAnzeigeQuerschnittSchaltbarkeitNba;
  }

  /**
   * Holen der HashMap mit dem Querschnittstatus für alle AnzeigenQuerschnitt aktuell.
   *
   * @return HashMap mit dem Querschnittstatus für alle AnzeigenQuerschnitt aktuell.
   */
  public HashMap<String, HashMap> getListeAnzeigeQuerschnittStatus()
  {
    return m_listeAnzeigeQuerschnittStatus;
  }

  /**
   * Holen der Datenverteilerverbindung.
   *
   * @return Datenverteilerverbindung.
   */
  public static ClientDavInterface getConnection()
  {
    return m_connection;
  }

  /**
   * Holen der HashMap mit den DirigentAktorTls mit Key DEs.
   *
   * @return HashMap mit den DirigentAktorTls mit Key DEs.
   */
  public HashMap<String, DirigentAktorTls> getListeDirigentAktorTlsKeyDes()
  {
    return m_listeDirigentAktorTlsKeyDes;
  }

  /**
   * Holen der HashMap mit den DirigentAktorTls mit Key Dirigent.
   *
   * @return HashMap mit den DirigentAktorTls mit Key Dirigent.
   */
  public HashMap<String, DirigentAktorTls> getListeDirigentAktorTlsKeyDirigent()
  {
    return m_listeDirigentAktorTlsKeyDirigent;
  }

  /**
   * Holen der HashMap mit der Attributgruppe atg.dirigentEigenschaftIst für alle Dirigenten aktuell.
   *
   * @return HashMap mit der Attributgruppe atg.dirigentEigenschaftIst für alle Dirigenten aktuell.
   */
  public HashMap<String, Data> getListeDirigentEigenschaftIst()
  {
    return m_listeDirigentEigenschaftIst;
  }

  /**
   * Holen der HashMap mit dem Dirigentenstatus für alle Dirigenten aktuell.
   *
   * @return HashMap mit dem Dirigentenstatus für alle Dirigenten aktuell.
   */
  public HashMap<String, HashMap> getListeDirigentStatus()
  {
    return m_listeDirigentStatus;
  }
}